# -*- coding: utf-8 -*-
#------------------------------------------------------------------------------
# file: $Id$
# auth: metagriffin <mg.github@metagriffin.net>
# date: 2013/09/04
# copy: (C) Copyright 2013-EOT metagriffin -- see LICENSE.txt
#------------------------------------------------------------------------------
# This software is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This software is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#------------------------------------------------------------------------------

import six, json
from .loader import configparser, Config, Section, isstr

list_types = (list, tuple, set)

#------------------------------------------------------------------------------
class Dumper(object):

  #----------------------------------------------------------------------------
  def __init__(self, jsonify=False):
    self.jsonify = jsonify

  #----------------------------------------------------------------------------
  def dumpfp(self, data, fp, jsonify_lists=False):
    if self.jsonify:
      fp.write(json.dumps(data))
      fp.write('\n')
      return
    if isinstance(data, Config):
      for section, options in data.items():
        # todo: ensure this escape mechanism is ok with ConfigParser...
        section = section.replace('\\', '\\\\').replace(']', '\\]')
        fp.write('[{}]\n'.format(section))
        self.dumpfp(options, fp)
      return
    if isinstance(data, Section):
      for option, value in data.items():
        fp.write(option)
        fp.write(' = ')
        self.dumpfp(data[option], fp, jsonify_lists=True)
      return
    if not jsonify_lists and isinstance(data, list_types):
      for item in data:
        fp.write(item)
        fp.write('\n')
      return
    if isstr(data):
      fp.write(data)
    else:
      fp.write(json.dumps(data))
    fp.write('\n')

default = Dumper()

#------------------------------------------------------------------------------
def dump(data, fp, dumper=None, *args, **kw):
  if dumper is None:
    if args or kw:
      dumper = Dumper(*args, **kw)
    else:
      dumper = default
  if isstr(fp):
    with open(fp, 'rb') as fp:
      return dump(data, fp, dumper=dumper)
  return dumper.dumpfp(data, fp)

#------------------------------------------------------------------------------
def dumps(data, *args, **kw):
  out = six.StringIO()
  dump(data, out, *args, **kw)
  return out.getvalue()

#------------------------------------------------------------------------------
# end of $Id$
#------------------------------------------------------------------------------



# -*- coding: utf-8 -*-
#------------------------------------------------------------------------------
# file: $Id$
# auth: metagriffin <mg.github@metagriffin.net>
# date: 2013/09/04
# copy: (C) Copyright 2013-EOT metagriffin -- see LICENSE.txt
#------------------------------------------------------------------------------
# This software is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This software is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#------------------------------------------------------------------------------

import six, json, iniherit, collections
from six.moves import configparser

DEFAULT_SECTION = configparser.DEFAULTSECT

#------------------------------------------------------------------------------
def isstr(obj):
  return isinstance(obj, six.string_types)

#------------------------------------------------------------------------------
class Config(collections.OrderedDict): pass
class Section(collections.OrderedDict): pass

#------------------------------------------------------------------------------
def sectkey(sect):
  if sect == DEFAULT_SECTION:
    return -100
  return sect

truthy = frozenset(('true', 'yes', 'on'))
falsy  = frozenset(('false', 'no', 'off'))

#------------------------------------------------------------------------------
class Loader(object):

  #----------------------------------------------------------------------------
  def __init__(self, inherit=True, case_sensitive=True,
               defaults=None, interpolate=True, fallback=True,
               jsonify=True):
    self.inherit      = inherit
    self.case         = case_sensitive
    self.defaults     = defaults
    self.interpolate  = interpolate
    self.jsonify      = jsonify
    self.fallback     = fallback

  #----------------------------------------------------------------------------
  def loadfp(self, fp):
    data = fp.read()
    attr = 'SafeConfigParser' if self.interpolate else 'RawConfigParser'
    mod  = iniherit if self.inherit else configparser
    ini  = getattr(mod, attr)(defaults=self.defaults)
    inir = getattr(mod, 'RawConfigParser')(defaults=self.defaults)
    ini.optionxform = str if self.case else str.lower
    inir.optionxform = str if self.case else str.lower
    ini.readfp(six.StringIO(data))
    inir.readfp(six.StringIO(data))
    def getopt(sect, opt):
      try:
        return ini.get(sect, opt)
      except Exception:
        if not self.interpolate or not self.fallback:
          raise
        return inir.get(sect, opt)
    ret = Config()
    ret[DEFAULT_SECTION] = Section()
    for opt in ini.defaults().keys():
      ret[DEFAULT_SECTION][opt] = getopt(DEFAULT_SECTION, opt)
    for section in sorted(ini.sections(), key=sectkey):
      ret[section] = Section()
      for opt in ini.options(section):
        ret[section][opt] = getopt(section, opt)
    if self.jsonify is False:
      return ret
    jret = Config()
    for sectname, options in ret.items():
      section = Section()
      for option, value in options.items():
        section[option] = self.parse(value)
      jret[sectname] = section
    return jret

  #----------------------------------------------------------------------------
  def parse(self, value):
    if value in truthy:
      return True
    if value in falsy:
      return False
    try:
      # todo: make this sensitive to self.jsonify being a range of values...
      #       eg. at jsonify==0.1, only parse scalars. at jsonify=0.2, also
      #       parse lists. at jsonify>=1.0 and is True, go all out.
      return json.loads(value)
    except ValueError:
      return value

default = Loader()

#------------------------------------------------------------------------------
def load(fp, loader=None, *args, **kw):
  if loader is None:
    if args or kw:
      loader = Loader(*args, **kw)
    else:
      loader = default
  if isstr(fp):
    with open(fp, 'rb') as fp:
      return load(fp, loader=loader)
  return loader.loadfp(fp)

#------------------------------------------------------------------------------
def loads(data, *args, **kw):
  return load(six.StringIO(data), *args, **kw)

#------------------------------------------------------------------------------
# end of $Id$
#------------------------------------------------------------------------------



# -*- coding: utf-8 -*-
#------------------------------------------------------------------------------
# file: $Id$
# auth: metagriffin <mg.github@metagriffin.net>
# date: 2013/09/04
# copy: (C) Copyright 2013-EOT metagriffin -- see LICENSE.txt
#------------------------------------------------------------------------------
# This software is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This software is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#------------------------------------------------------------------------------

import six
from .loader import DEFAULT_SECTION, configparser, Config, Section, isstr, sectkey

#------------------------------------------------------------------------------
def sift(data,
         list_sections=False, list_keys=False,
         sections=None, keys=None):

  if list_sections:
    return sorted(data.keys(), key=sectkey)

  if list_keys:
    if isstr(sections):
      try:
        return sorted(data[sections].keys())
      except KeyError:
        raise configparser.NoSectionError(section=sections)
    try:
      ret = Config()
      for section in sorted(sections or data.keys(), key=sectkey):
        ret[section] = sorted(data[section].keys())
      return ret
    except KeyError as err:
      raise configparser.NoSectionError(section=err.message)

  if isstr(sections):
    if sections not in data:
      raise configparser.NoSectionError(section=sections)
    section = data[sections]
    if isstr(keys):
      try:
        return section[keys]
      except KeyError:
        raise configparser.NoOptionError(section=sections, option=keys)
    if keys is None:
      return section
    try:
      ret = Section()
      for key in sorted(keys):
        ret[key] = section[key]
      return ret
    except KeyError:
      raise configparser.NoOptionError(section=sections, option=keys)

  ret = Config()
  for sectname in sorted(sections or data.keys(), key=sectkey):
    if sectname not in data:
      raise configparser.NoSectionError(section=sectname)
    section = data[sectname]
    ret[sectname] = Section()
    for key in sorted(keys or section.keys()):
      if key not in section:
        raise configparser.NoOptionError(section=sectname, option=key)
      ret[sectname][key] = section[key]
  return ret


#------------------------------------------------------------------------------
# end of $Id$
#------------------------------------------------------------------------------



# -*- coding: utf-8 -*-
#------------------------------------------------------------------------------
# file: $Id$
# auth: metagriffin <mg.github@metagriffin.net>
# date: 2013/09/04
# copy: (C) Copyright 2013-EOT metagriffin -- see LICENSE.txt
#------------------------------------------------------------------------------
# This software is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This software is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#------------------------------------------------------------------------------

import unittest, six

from . import loader, sifter, dumper

#------------------------------------------------------------------------------
class TestIniget(unittest.TestCase):

  #----------------------------------------------------------------------------
  def test_load(self):
    ini = '[DEFAULT]\na = b\n[sect1]\nc = true\n'
    chk = {'DEFAULT': {'a': 'b'}, 'sect1': {'a': 'b', 'c': True}}
    out = loader.load(six.StringIO(ini))
    self.assertEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_loads(self):
    ini = '[DEFAULT]\na = b\n[sect1]\nc = true\nC = e\n'
    chk = {'DEFAULT': {'a': 'b'}, 'sect1': {'a': 'b', 'c': True, 'C': 'e'}}
    out = loader.loads(ini)
    self.assertEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_loads_icase(self):
    ini = '[DEFAULT]\na = b\n[sect1]\nc = true\nC = e\n'
    chk = {'DEFAULT': {'a': 'b'}, 'sect1': {'a': 'b', 'c': 'e'}}
    out = loader.loads(ini, case_sensitive=False)
    self.assertEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_loads_moreTruthy(self):
    ini = '[DEFAULT]\na = b\nb = true\nc = on\nd = yes\n'
    chk = {'DEFAULT': {'a': 'b', 'b': True, 'c': True, 'd': True}}
    out = loader.loads(ini)
    self.assertEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_loads_moreFalsy(self):
    ini = '[DEFAULT]\na = b\nb = false\nc = off\nd = no\n'
    chk = {'DEFAULT': {'a': 'b', 'b': False, 'c': False, 'd': False}}
    out = loader.loads(ini)
    self.assertEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_loads_defaultsMissing(self):
    ini = '[DEFAULT]\na = b\n[sect1]\nc = no-such-%(here)s-data\n'
    with self.assertRaises(loader.configparser.InterpolationMissingOptionError) as cm:
      out = loader.loads(ini, fallback=False)
    self.assertEqual(cm.exception.reference, 'here')

  #----------------------------------------------------------------------------
  def test_loads_defaults(self):
    ini = '[DEFAULT]\na = b\n[sect1]\nc = no-such-%(here)s-data\n'
    chk = {'DEFAULT': {'a': 'b', 'here': 'str'},
           'sect1': {'a': 'b', 'c': 'no-such-str-data', 'here': 'str'}}
    out = loader.loads(ini, defaults=dict(here='str'))
    self.assertEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_evaluates_default_section(self):
    ini = '[DEFAULT]\na = b\nc = %(a)s\n'
    chk = {'DEFAULT': {'a': 'b', 'c': 'b'}}
    out = loader.loads(ini)
    self.assertEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_bad_interpolation_value(self):
    ini = '[logger]\ndatefmt = %Y-%m-%d\n'
    chk = {'DEFAULT': {}, 'logger': {'datefmt': '%Y-%m-%d'}}
    out = loader.loads(ini)
    self.assertEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_loads_nointerpolate(self):
    ini = '[DEFAULT]\na = b\n[sect1]\nc = no-such-%(here)s-data\n'
    chk = {'DEFAULT': {'a': 'b'}, 'sect1': {'a': 'b', 'c': 'no-such-%(here)s-data'}}
    out = loader.loads(ini, interpolate=False)
    self.assertEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_load_jsonify(self):
    ini = '[DEFAULT]\na = b\n[sect1]\nc = true\ne = true\nf = 12\n'
    chk = {'DEFAULT': {'a': 'b'}, 'sect1': {'a': 'b', 'c': True, 'e': True, 'f': 12}}
    out = loader.load(six.StringIO(ini))
    self.assertEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_load_nojsonify(self):
    ini = '[DEFAULT]\na = b\n[sect1]\nc = true\ne = true\nf = 12\n'
    chk = {'DEFAULT': {'a': 'b'}, 'sect1': {'a': 'b', 'c': 'true', 'e': 'true', 'f': '12'}}
    out = loader.load(six.StringIO(ini), jsonify=False)
    self.assertEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_sift_sections(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect1]\nc = true\n')
    chk = ['DEFAULT', 'sect1']
    out = sifter.sift(ini, list_sections=True)
    self.assertEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_sift_keysAllSections(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect1]\nc = true\n')
    chk = {'DEFAULT': ['a'], 'sect1': ['a', 'c']}
    out = sifter.sift(ini, list_keys=True)
    self.assertEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_sift_keysOneSectionStr(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect1]\nc = true\n[sect2]\ne = f\n')
    chk = ['a', 'c']
    out = sifter.sift(ini, list_keys=True, sections='sect1')
    self.assertEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_sift_keysOneSectionList(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect1]\nc = true\n[sect2]\ne = f\n')
    chk = {'sect1': ['a', 'c']}
    out = sifter.sift(ini, list_keys=True, sections=['sect1'])
    self.assertEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_sift_keysMultipleSections(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect1]\nc = true\n[sect2]\ne = f\n')
    chk = {'sect1': ['a', 'c'], 'sect2': ['a', 'e']}
    out = sifter.sift(ini, list_keys=True, sections=['sect1', 'sect2'])
    self.assertEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_sift_getSectKeyStr(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect1]\nc = true\n[sect2]\ne = f\n')
    chk = 'f'
    out = sifter.sift(ini, sections='sect2', keys='e')
    self.assertEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_sift_noSectKey(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect1]\nc = true\n[sect2]\ne = f\n')
    with self.assertRaises(loader.configparser.NoSectionError) as cm:
      out = sifter.sift(ini, sections='no-such-sect', keys='e')
    self.assertIn('no-such-sect', cm.exception.message)

  #----------------------------------------------------------------------------
  def test_sift_sectNoKey(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect1]\nc = true\n[sect2]\ne = f\n')
    with self.assertRaises(loader.configparser.NoOptionError) as cm:
      out = sifter.sift(ini, sections='sect1', keys='no-such-key')
    self.assertIn('no-such-key', cm.exception.message)

  #----------------------------------------------------------------------------
  def test_sift_getSectKeyListOne(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect1]\nc = true\n[sect2]\ne = f\n')
    chk = {'c': True}
    out = sifter.sift(ini, sections='sect1', keys=['c'])
    self.assertEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_sift_getSectKeyListMultiple(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect1]\nc = true\n[sect2]\ne = f\n')
    chk = {'a': 'b', 'c': True}
    out = sifter.sift(ini, sections='sect1', keys=['a', 'c'])
    self.assertEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_sift_getSectsKey(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect1]\nc = true\n[sect2]\ne = f\n')
    chk = {'sect1': {'a': 'b'}, 'sect2': {'a': 'b'}}
    out = sifter.sift(ini, sections=['sect1', 'sect2'], keys=['a'])
    self.assertEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_sift_getSectsKeys(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect1]\nc = true\n[sect2]\nc = false\ne = f\n')
    chk = {'sect1': {'a': 'b', 'c': True}, 'sect2': {'a': 'b', 'c': False}}
    out = sifter.sift(ini, sections=['sect1', 'sect2'], keys=['a', 'c'])
    self.assertEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_sift_getSectsKeysNoSect(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect1]\nc = true\n[sect2]\nc = true2\ne = f\n')
    chk = {'sect1': {'a': 'b', 'c': 'd'}, 'sect2': {'a': 'b', 'c': 'd2'}}
    with self.assertRaises(loader.configparser.NoSectionError) as cm:
      out = sifter.sift(ini, sections=['sect1', 'sect2', 'no-such-sect'], keys=['a', 'c'])
    self.assertIn('no-such-sect', cm.exception.message)

  #----------------------------------------------------------------------------
  def test_sift_getSectsKeysNoKey(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect1]\nc = true\n[sect2]\ne = f\n')
    chk = {'sect1': {'a': 'b', 'c': 'd'}, 'sect2': {'a': 'b', 'c': 'd2'}}
    with self.assertRaises(loader.configparser.NoOptionError) as cm:
      out = sifter.sift(ini, sections=['sect1', 'sect2'], keys=['a', 'no-such-key'])
    self.assertIn('no-such-key', cm.exception.message)
    self.assertIn('sect1', cm.exception.message)

  #----------------------------------------------------------------------------
  def test_dump(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect1]\nc = true\n[sect2]\nc = true2\ne = f\n')
    ini = sifter.sift(ini, sections='sect1', keys='c')
    out = six.StringIO()
    dumper.dump(ini, out)
    chk = 'true\n'
    self.assertMultiLineEqual(out.getvalue(), chk)

  #----------------------------------------------------------------------------
  def test_dumps(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect1]\nc = true\n[sect2]\nc = true2\ne = f\n')
    ini = sifter.sift(ini, sections='sect1', keys='c')
    out = dumper.dumps(ini)
    chk = 'true\n'
    self.assertMultiLineEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_dumps_listSects(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect1]\nc = true\n[sect2]\nc = true2\ne = f\n')
    ini = sifter.sift(ini, list_sections=True)
    out = dumper.dumps(ini)
    chk = 'DEFAULT\nsect1\nsect2\n'
    self.assertMultiLineEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_dumps_listSectsJson(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect1]\nc = true\n[sect2]\nc = true2\ne = f\n')
    ini = sifter.sift(ini, list_sections=True)
    out = dumper.dumps(ini, jsonify=True)
    chk = '["DEFAULT", "sect1", "sect2"]\n'
    self.assertMultiLineEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_dumps_listKeys(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect1]\nc = true\n[sect2]\nc = true2\ne = f\n')
    ini = sifter.sift(ini, list_keys=True)
    out = dumper.dumps(ini)
    chk = '[DEFAULT]\na\n[sect1]\na\nc\n[sect2]\na\nc\ne\n'
    self.assertMultiLineEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_dumps_listKeysJson(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect1]\nc = true\n[sect2]\nc = true2\ne = f\n')
    ini = sifter.sift(ini, list_keys=True)
    out = dumper.dumps(ini, jsonify=True)
    chk = '{"DEFAULT": ["a"], "sect1": ["a", "c"], "sect2": ["a", "c", "e"]}\n'
    self.assertMultiLineEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_dumps_listKeysJson_ordering(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect2]\ne = f\nc = true2\n[sect1]\nc = true\n')
    ini = sifter.sift(ini, list_keys=True)
    out = dumper.dumps(ini, jsonify=True)
    chk = '{"DEFAULT": ["a"], "sect1": ["a", "c"], "sect2": ["a", "c", "e"]}\n'
    self.assertMultiLineEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_dumps_section(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect1]\nd = foo\nc = true\n')
    ini = sifter.sift(ini, sections='sect1')
    out = dumper.dumps(ini)
    chk = 'd = foo\nc = true\na = b\n'
    self.assertMultiLineEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_dumps_sectionJson(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect1]\nc = true\nd = foo\n')
    ini = sifter.sift(ini, sections='sect1')
    out = dumper.dumps(ini, jsonify=True)
    chk = '{"c": true, "d": "foo", "a": "b"}\n'
    self.assertMultiLineEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_dumps_value(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect2]\ne = f\nc = true2\n[sect1]\nc = true\n')
    ini = sifter.sift(ini, sections='sect2', keys='c')
    out = dumper.dumps(ini)
    chk = 'true2\n'
    self.assertMultiLineEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_dumps_valueJson(self):
    ini = loader.loads('[DEFAULT]\na = b\n[sect2]\ne = f\nc = true2\n[sect1]\nc = true\n')
    ini = sifter.sift(ini, sections='sect2', keys='c')
    out = dumper.dumps(ini, jsonify=True)
    chk = '"true2"\n'
    self.assertMultiLineEqual(out, chk)


#------------------------------------------------------------------------------
# end of $Id$
#------------------------------------------------------------------------------



# -*- coding: utf-8 -*-
#------------------------------------------------------------------------------
# file: $Id$
# auth: metagriffin <mg.github@metagriffin.net>
# date: 2013/08/20
# copy: (C) Copyright 2014-EOT metagriffin -- see LICENSE.txt
#------------------------------------------------------------------------------
# This software is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This software is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#------------------------------------------------------------------------------

import sys, argparse, json
from . import loader, sifter, dumper
from .i18n import _

#------------------------------------------------------------------------------
def main(argv=None):

  cli = argparse.ArgumentParser(
    description = _(
      'Simple tool to extract, normalize, and JSONify values from an'
      ' iniherit-enabled "INI" configuration file.'),
    )

  cli.add_argument(
    _('-s'), _('--section'), metavar=_('SECTION'),
    dest='sections', action='append', default=[],
    help=_('specify additional sections to extract from; can be'
           ' specified multiple times to add multiple sections'))

  cli.add_argument(
    _('-C'), _('--no-case'),
    dest='case', action='store_false', default=True,
    help=_('handle option names case insensitively'))

  cli.add_argument(
    _('-I'), _('--no-inherit'),
    dest='inherit', action='store_false', default=True,
    help=_('disable processing of "%%inherit" directives'))

  cli.add_argument(
    _('-E'), _('--no-expansion'),
    dest='interpolate', action='store_false', default=True,
    help=_('disable ConfigParser option expansion ("interpolation")'))

  cli.add_argument(
    _('-F'), _('--no-fallback'),
    dest='fallback', action='store_false', default=True,
    help=_('disable falling back to non-interpolated option expansion'))

  cli.add_argument(
    _('-K'), _('--list-options'),
    dest='list_keys', action='store_true', default=False,
    help=_('list the option names only, not the values'))

  cli.add_argument(
    _('-S'), _('--list-sections'),
    dest='list_sections', action='store_true', default=False,
    help=_('list the section names only'))

  cli.add_argument(
    _('-J'), _('--json-parse'),
    dest='json_parse', action='store_true', default=False,
    help=_('if option values are JSON-parseable, parse as such;'
           ' additionally, the following are interpreted as boolean'
           ' values: {}', list(reversed(sorted(list(loader.truthy)
                                               + list(loader.falsy))))))

  cli.add_argument(
    _('-j'), _('--json-output'),
    dest='json_output', action='store_true', default=False,
    help=_('render the output using JSON syntax'))

  cli.add_argument(
    _('-d'), _('--defaults'), metavar=_('JSON'),
    dest='defaults', action='store',
    help=_('set the ConfigParser default values from this JSON-parsed'
           ' dictionary'))

  cli.add_argument(
    _('-r'), _('--raw'),
    dest='raw', action='store_true', default=False,
    help=_('don\'t do anything fancy: show exactly what ConfigParser'
           ' interprets (requires exactly one section and one option);'
           ' note that options "--no-case", "--no-inherit", "--no-expansion"'
           ' and "--defaults" are still honored, but "--json-parse" is not'))

  cli.add_argument(
    'config', metavar=_('FILENAME'),
    help=_('path to INI configuration file; if exactly "-", the'
           ' configuration is read from STDIN'))

  cli.add_argument(
    'section', metavar=_('SECTION'),
    nargs='?',
    help=_('configuration section to extract from; if not specified or'
           ' exactly "-", select all sections'))

  cli.add_argument(
    'keys', metavar=_('OPTION'),
    nargs='*',
    help=_('configuration option to extract; if not specified or'
           ' exactly "-", list all option/value pairs in the selected'
           ' section(s)'))

  options = cli.parse_args(args=argv)

  # todo: confirm that empty section/key names are indeed not valid...

  # clean up section names
  if options.section == '-':
    options.section = None
  options.sections.append(options.section)
  options.sections = filter(None, options.sections)
  if not options.sections:
    options.sections = None
  elif len(options.sections) == 1:
    options.sections = options.sections[0]

  # clean up key names
  if options.keys == ['-']:
    options.keys = None
  options.keys = filter(None, options.keys)
  if not options.keys:
    options.keys = None
  elif len(options.keys) == 1:
    options.keys = options.keys[0]

  if options.config == '-':
    options.config = sys.stdin

  if options.defaults:
    try:
      options.defaults = json.loads(options.defaults)
    except Exception as err:
      cli.error(_('"--defaults" must specify a JSON dictionary;'
                  ' parsing error: ', options.defaults) + str(err))

  if options.raw:
    if not loader.isstr(options.sections) or not loader.isstr(options.keys):
      cli.error(_('raw mode ("--raw") requires exactly one section and'
                  ' one option to be specified'))
    options.json_parse = False

  result = loader.load(
    options.config,
    inherit=options.inherit, case_sensitive=options.case,
    interpolate=options.interpolate, defaults=options.defaults,
    fallback=options.fallback,
    jsonify=options.json_parse)

  try:
    if options.raw:
      if options.sections not in result:
        raise loader.configparser.NoSectionError(section=options.sections)
      section = result[options.sections]
      if options.keys not in section:
        raise loader.configparser.NoOptionError(section=options.sections,
                                                option=options.keys)
      print repr(section[options.keys])
      return 0
    result = sifter.sift(
      result,
      list_keys=options.list_keys, list_sections=options.list_sections,
      sections=options.sections, keys=options.keys)
  except loader.configparser.NoSectionError as err:
    print >>sys.stderr, _('ERROR: no such section "{}"', err.section)
    return 20
  except loader.configparser.NoOptionError as err:
    print >>sys.stderr, _('ERROR: no such option "{}" in section "{}"',
                          err.option, err.section)
    return 21

  dumper.dump(
    result, sys.stdout,
    jsonify=options.json_output)
  return 0

#------------------------------------------------------------------------------
# end of $Id$
#------------------------------------------------------------------------------
